/***************************************************************************
                          CdboDialogBase.h  -  description
                             -------------------
    begin                : Tue Jul 17 2001
    copyright            : (C) 2001 by Alex Podolsky
    email                : alexpod@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; see the file COPYING.  If not, write to
	the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA 02111-1307, USA.

***************************************************************************/

#ifndef CDBODIALOGBASE_H
#define CDBODIALOGBASE_H

#include <qvariant.h>
#include <kdialog.h>
#include <dcopobject.h>

#include "CdboAction.h"

class KProcess;
class KConfig;
class KLibFactory;

/**
 * Base Dialog for all cdbakeoven's action dialogs. Provides basic infrastructure
 * and means of commucation with Actions.
 *
 * @see CdboAction
 * @author Alex Podolsky <alexpod@users.sourceforge.net>
*/

class CdboDialogBase: public KDialog, public virtual DCOPObject{

Q_OBJECT

public:
	/**
     * Constructor.
     *
     * Takes the same arguments as @ref KDialog.
     */
    CdboDialogBase( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
	~CdboDialogBase();
	/**
     */
	bool process(const QCString &fun, const QByteArray &data, QCString&, QByteArray &);
	void debugCommands(){ DEBUG_COMMANDS = "TRUE"; }
	void debugOutput(){ DEBUG_OUTPUT = "TRUE"; }
	virtual void setCurrentTarget(const QString &/*device*/){}

public slots:
	/**
     * Read configuration from config file.
     * Only calls @ref reset(), override to add functionality.
     *
     * @see reset()
     */
	virtual void readOptions();
	/**
     * Block buttons during execution.
     * Empty, reimplement to add functionality
     */
	virtual void blockBtns(bool){}

	void detectSlot();
	void settingsSlot();

signals:
	void dialogDone(KDialog*);

protected slots:
	/**
     * Save curent settings.
     * Empty, reimplement to add functionality
     */
	virtual void saveOptions() {};
	/**
     * Called at the end of action's execution. It is connected to
     * current action's processDone() signal.
     * Only calls @ref reset(), override to add functionality.
     *
	* @ param success Will be false if action failed.
     *
     * @see reset()
     * @see CdboAction
     */
	virtual void taskDone(bool success);
	/**
     * Called if action failed to start.It is connected to
     * current action's failedToLaunchProcess() signal.
     * Only calls @ref reset(), override to add functionality.
     *
     * @see reset()
     * @see CdboAction
     */
	virtual void taskFailedToStart();
	/**
     * Called if action was cancelled, but not from this dialog. It is connected to
     * current action's processCancelled() signal.
     * Only calls @ref reset(), override to add functionality.
     *
     * @see reset()
     * @see CdboAction
     */
	virtual void taskCancelled();
	/**
     * Connect this slot to cancel button. If dialog is running action,
     * it will call tryToCancel() on that action.
     *
     * @see CdboAction
     */
	virtual void cancelSlot();
	/**
     * Connect this slot to quit button. If dialog is running action,
     * it will call tryToCancel() on that action.
     *
     * @see CdboAction
     */
	virtual void quitSlot();
	/**
     * If dialog is running action, it will call tryToCancel() on that action,
     * if that returns true, will close this dialog.
     *
     * @see CdboAction
     */
	virtual void keyPressEvent(QKeyEvent *e);
	/**
     * If dialog is running action, it will call tryToCancel() on that action,
     * if that returns true, will close this dialog.
     *
     * @see CdboAction
     */
	virtual void closeEvent(QCloseEvent *e);
	virtual void accept ();
	virtual void reject ();

	void emitDialogDone() { emit dialogDone(this); }

private slots:
	void kcmshellDone(KProcess *proc);

protected:
	/**
     * Empty, reimplement to add functionality
     */
	virtual void initBtns() {};
	/**
     * Empty, reimplement to add functionality
     */
	virtual void stopForwardTimer() {};
	/**
     * Empty, reimplement to add functionality
     */
	virtual void startForwardTimer() {};
	/**
     * Empty, reimplement to add functionality
     */
	virtual void clearOutput() {};
	/**
     * Call this from the constractor at least. If override, do not forget to call supper.
     */
	virtual void init();
	/**
     * Returns the widget that will be connected to action's output(), percent(),
	* and percentInfo() signals. Returns 0 by dafault, so those signals will be ignored.
	* @see CdboAction
     */
	virtual QWidget* outputWidget() { return 0; }
	/**
     * Returns the widget that will be connected to action's status() signal.
     * Returns 0 by dafault, so that signal will be ignored.
     * @see CdboAction
     */
	virtual QWidget* statusWidget() { return 0; }
	/**
     * Resets settimgs after action is done. By dafault: stops timer, sets mDoingSomething
     * to false and makes currAction 0.
     */
	virtual void reset();
	//virtual void cleanUp() {/*empty*/;}

	/**
     * Returns true if dialog is currently running some action.
     * Manipulate it with mDoingSomething.
     *
     * @return true if busy.
     */
	virtual bool doingSomething();
	/**
     * Reinitializes configuration file. Will use cdbakeoven's one
     * by default, override to point to a different config.
     */
	virtual void resetConfig();
	/**
     * Creates new CdboAction with classname = @p type, and makes it
     * currAction fot this dialog.
     *
     * @ param type the type (classname) of CdboAction to create.
     *
     * @return false if failed to create this action.
     * @see CdboAction
     */
	bool createAction(const QString type, const QString libName = "libcdboactions");
	/**
     * Pops up kcmshell.
     *
     * @ param type the types configuration dialog to pass to the shell.
     * May be more than one.
     */
	void popupSettings(QString type);
	QString timeStamp();

	void addActionParam(QString name, QString *value);
	bool removeActionParam(QString name);
	void clearActionParams();

	bool launchCurrAction();
	bool isCurrAction(QString wanted);

	QString settingTypes;

	KConfig *config;

	bool mDoingSomething;
	QString mActionType;


private:
	CdboParamsDict params;
	CdboAction *currAction;
	QString DEBUG_COMMANDS;
	QString DEBUG_OUTPUT;

};
#endif
